@@ -43,6 +43,7 @@ typedef struct {
   GSocketConnectable *identity;
   GSocketAddress *address;
   GTlsAuthenticationMode auth_mode;
+  gboolean rehandshake;
   GTlsCertificateFlags accept_flags;
 } TestConnection;
 
@@ -75,22 +76,28 @@ teardown_connection (TestConnection *test, gconstpointer data)
   if (test->server_connection)
     {
       g_assert (G_IS_TLS_SERVER_CONNECTION (test->server_connection));
+      g_object_add_weak_pointer (G_OBJECT (test->server_connection),
+				 (gpointer *)&test->server_connection);
       g_object_unref (test->server_connection);
-      g_assert (!G_IS_TLS_SERVER_CONNECTION (test->server_connection));
+      g_assert (test->server_connection == NULL);
     }
 
   if (test->client_connection)
     {
       g_assert (G_IS_TLS_CLIENT_CONNECTION (test->client_connection));
+      g_object_add_weak_pointer (G_OBJECT (test->client_connection),
+				 (gpointer *)&test->client_connection);
       g_object_unref (test->client_connection);
-      g_assert (!G_IS_TLS_SERVER_CONNECTION (test->client_connection));
+      g_assert (test->client_connection == NULL);
     }
 
   if (test->database)
     {
       g_assert (G_IS_TLS_DATABASE (test->database));
+      g_object_add_weak_pointer (G_OBJECT (test->database),
+				 (gpointer *)&test->database);
       g_object_unref (test->database);
-      g_assert (!G_IS_TLS_DATABASE (test->database));
+      g_assert (test->database == NULL);
     }
 
   g_object_unref (test->address);
@@ -106,6 +113,29 @@ on_accept_certificate (GTlsClientConnection *conn, GTlsCertificate *cert,
   return errors == test->accept_flags;
 }
 
+static void on_output_write_finish (GObject        *object,
+				    GAsyncResult   *res,
+				    gpointer        user_data);
+
+static void
+on_rehandshake_finish (GObject        *object,
+		       GAsyncResult   *res,
+		       gpointer        user_data)
+{
+  TestConnection *test = user_data;
+  GError *error = NULL;
+  GOutputStream *stream;
+
+  g_tls_connection_handshake_finish (G_TLS_CONNECTION (object), res, &error);
+  g_assert_no_error (error);
+
+  stream = g_io_stream_get_output_stream (test->server_connection);
+  g_output_stream_write_async (stream, TEST_DATA + TEST_DATA_LENGTH / 2,
+			       TEST_DATA_LENGTH / 2,
+                               G_PRIORITY_DEFAULT, NULL,
+                               on_output_write_finish, test);
+}
+
 static void
 on_output_close_finish (GObject        *object,
                         GAsyncResult   *res,
@@ -123,9 +153,19 @@ on_output_write_finish (GObject        *object,
 {
   TestConnection *test = user_data;
   GError *error = NULL;
+
   g_output_stream_write_finish (G_OUTPUT_STREAM (object), res, &error);
   g_assert_no_error (error);
 
+  if (test->rehandshake)
+    {
+      test->rehandshake = FALSE;
+      g_tls_connection_handshake_async (G_TLS_CONNECTION (test->server_connection),
+					G_PRIORITY_DEFAULT, NULL,
+					on_rehandshake_finish, test);
+      return;
+    }
+
   g_output_stream_close_async (G_OUTPUT_STREAM (object), G_PRIORITY_DEFAULT, NULL,
                                on_output_close_finish, test);
 }
@@ -161,7 +201,8 @@ on_incoming_connection (GSocketService     *service,
 
   stream = g_io_stream_get_output_stream (test->server_connection);
 
-  g_output_stream_write_async (stream, TEST_DATA, TEST_DATA_LENGTH,
+  g_output_stream_write_async (stream, TEST_DATA,
+			       test->rehandshake ? TEST_DATA_LENGTH / 2 : TEST_DATA_LENGTH,
                                G_PRIORITY_DEFAULT, NULL,
                                on_output_write_finish, test);
   return FALSE;
@@ -324,6 +365,14 @@ test_client_auth_connection (TestConnection *test,
   g_main_loop_run (test->loop);
 }
 
+static void
+test_client_auth_rehandshake (TestConnection *test,
+			      gconstpointer   data)
+{
+  test->rehandshake = TRUE;
+  test_client_auth_connection (test, data);
+}
+
 static void
 test_connection_no_database (TestConnection *test,
                              gconstpointer   data)
@@ -406,8 +455,9 @@ test_create_destroy_certificate_pem (TestCertificate *test, gconstpointer data)
   g_assert_cmpstr (pem, ==, test->pem);
   g_free (pem);
 
+  g_object_add_weak_pointer (G_OBJECT (cert), (gpointer *)&cert);
   g_object_unref (cert);
-  g_assert (!G_IS_TLS_CERTIFICATE (cert));
+  g_assert (cert == NULL);
 }
 
 static void
@@ -432,8 +482,9 @@ test_create_destroy_certificate_der (TestCertificate *test, gconstpointer data)
   g_assert (memcmp (der->data, test->der->data, der->len) == 0);
   g_byte_array_unref (der);
 
+  g_object_add_weak_pointer (G_OBJECT (cert), (gpointer *)&cert);
   g_object_unref (cert);
-  g_assert (!G_IS_TLS_CERTIFICATE (cert));
+  g_assert (cert == NULL);
 }
 
 static void
@@ -460,15 +511,17 @@ test_create_certificate_with_issuer (TestCertificate   *test,
   g_assert_no_error (error);
   g_assert (G_IS_TLS_CERTIFICATE (cert));
 
+  g_object_add_weak_pointer (G_OBJECT (issuer), (gpointer *)&issuer);
   g_object_unref (issuer);
-  g_assert (G_IS_TLS_CERTIFICATE (issuer));
+  g_assert (issuer != NULL);
 
   check = g_tls_certificate_get_issuer (cert);
   g_assert (check == issuer);
 
+  g_object_add_weak_pointer (G_OBJECT (cert), (gpointer *)&cert);
   g_object_unref (cert);
-  g_assert (!G_IS_TLS_CERTIFICATE (cert));
-  g_assert (!G_IS_TLS_CERTIFICATE (issuer));
+  g_assert (cert == NULL);
+  g_assert (issuer == NULL);
 }
 
 /* -----------------------------------------------------------------------------
@@ -512,16 +565,22 @@ teardown_verify (TestVerify      *test,
                  gconstpointer    data)
 {
   g_assert (G_IS_TLS_CERTIFICATE (test->cert));
+  g_object_add_weak_pointer (G_OBJECT (test->cert),
+			     (gpointer *)&test->cert);
   g_object_unref (test->cert);
-  g_assert (!G_IS_TLS_CERTIFICATE (test->cert));
+  g_assert (test->cert == NULL);
 
   g_assert (G_IS_TLS_CERTIFICATE (test->anchor));
+  g_object_add_weak_pointer (G_OBJECT (test->anchor),
+			     (gpointer *)&test->anchor);
   g_object_unref (test->anchor);
-  g_assert (!G_IS_TLS_CERTIFICATE (test->anchor));
+  g_assert (test->anchor == NULL);
 
   g_assert (G_IS_TLS_DATABASE (test->database));
+  g_object_add_weak_pointer (G_OBJECT (test->database),
+			     (gpointer *)&test->database);
   g_object_unref (test->database);
-  g_assert (!G_IS_TLS_DATABASE (test->database));
+  g_assert (test->database == NULL);
 }
 
 static void
@@ -832,8 +891,10 @@ teardown_file_database (TestFileDatabase *test,
                         gconstpointer     data)
 {
   g_assert (G_IS_TLS_DATABASE (test->database));
+  g_object_add_weak_pointer (G_OBJECT (test->database),
+			     (gpointer *)&test->database);
   g_object_unref (test->database);
-  g_assert (!G_IS_TLS_DATABASE (test->database));
+  g_assert (test->database == NULL);
 
   g_free (test->path);
 }
@@ -945,6 +1006,8 @@ main (int   argc,
               setup_connection, test_verified_connection, teardown_connection);
   g_test_add ("/tls/connection/client-auth", TestConnection, NULL,
               setup_connection, test_client_auth_connection, teardown_connection);
+  g_test_add ("/tls/connection/client-auth-rehandshake", TestConnection, NULL,
+              setup_connection, test_client_auth_rehandshake, teardown_connection);
   g_test_add ("/tls/connection/no-database", TestConnection, NULL,
               setup_connection, test_connection_no_database, teardown_connection);
 
